"""" Command line interface for the bot"""
import logging
import os
import sys

import click
import colorama
from pyfiglet import figlet_format
from termcolor import colored

project_dir = os.path.dirname(os.path.dirname(os.path.realpath(__file__)))
sys.path.append(project_dir)

from src.config.env import EnvConfig as Conf

logging.basicConfig(
    format=Conf.LOG_FORMAT, level=Conf.LOG_LEVEL
)
colorama.init()


@click.command()
@click.option('--poll', is_flag=True, help='Start the bot with polling method')
@click.option('--crawl', is_flag=True, help='Start news crawling')
@click.option('--start', is_flag=True, help='Start the bot webhook')
@click.option('--update', is_flag=True, help='Update database')
def handle_commands(poll, crawl, start, update):
    """Main method for handling commands through CLI"""
    if not Conf.BOT_DB or not Conf.BOT_TOKEN:
        stdout("ERROR: please set BOT_DATABASE and BOT_TOKEN to start the bot! ", color="red")
        return
    bot_db = Conf.BOT_DB
    stdout("\nTabayyun Bot", color="blue", figlet_flag=True)
    if poll:
        stdout("Starting polling messages...", color="green")
        try:
            from src.view import telegram_bot
            telegram_bot.start_bot()
            stdout("Server completed", color="green")
        except Exception as exc:
            stdout("Starting server failed! : {}".format(exc), color="red")

    if crawl:
        crawling_page = int(input("Masukan jumlah artikel yang mau di download : "))
        try:
            from src.service import crawler
            from src.dataset.database import TabayyunDB
            stdout("Applying schema...", color="yellow")
            stdout(f"Using database {bot_db}", color="yellow")
            db_conn = TabayyunDB(sqlite_db=Conf.BOT_DB)
            db_conn.apply_schema()
            stdout("Start crawling...", color="yellow")
            crawler.start(crawling_page)
            stdout("Crawling completed!", color="green")
        except Exception as exc:
            stdout("Crawling failed! : {}".format(exc), color="red")

    if start:
        if not Conf.WEBHOOK_URL:
            stdout("ERROR: WEBHOOK_URL needs to be set for starting webhook server! ", color="red")
        if not Conf.WEBHOOK_HOST:
            stdout("WARNING: WEBHOOK_HOST is not set, set default value : 0.0.0.0", color="yellow")
        if not Conf.WEBHOOK_PORT:
            stdout("WARNING: WEBHOOK_PORT is not set, set default value : 8443", color="yellow")
        else:
            stdout(f"WEBHOOK_URL : {os.environ['WEBHOOK_URL']}", color="yellow")
        stdout("Starting Tabayyun webhook server...", color="green")
        from src.view import telegram_webhook
        telegram_webhook.start()
        try:
            from src.view import telegram_webhook
            telegram_webhook.start()
        except Exception as e:
            stdout("Starting server failed! : {}".format(e), color="red")


def stdout(string: str, color: str, font: str = "slant", figlet_flag: bool = False):
    """ A helper method to print logs to stdout """
    if not figlet_flag:
        logging.info(colored(string, color))
    else:
        logging.info(colored(figlet_format(string, font=font), color))


if __name__ == "__main__":
    handle_commands()



import logging

from src.config.env import EnvConfig as Conf
from src.dataclasses.model import BotUser
from src.dataclasses.model import Chat
from src.dataset.database import TabayyunDB
from src.service import crawler
from src.service.text_similarity import get_cosine_sim

logging.basicConfig(
    format=Conf.LOG_FORMAT, level=Conf.LOG_LEVEL
)


class Server:
    """ The API server """

    def handler(self, chat: Chat):
        """ Method for handing chats"""
        # get text similarity
        similarity = get_cosine_sim(chat.str_message)

        # process reply
        reply: str
        if similarity.get('conclusion'):
            fact = similarity['fact']
            classification = similarity['classification'].upper()
            conclusion = similarity['conclusion']
            references = "\n".join(similarity['references'])
            reply = f"Kategori : {classification} \n\nFakta : \n{fact} \n\nKesimpulan :\n{conclusion} \n\nArtikel terkait:\n{references}"
        elif similarity.get('references'):
            fact = similarity['fact']
            references = "\n".join(similarity['references'])
            reply = f"Konten : \n{chat.str_message} \n\nAnalisa :\n{fact} \n\nArtikel terkait:\n{references}"
        else:
            fact = "Tidak ditemukan artikel terkait konten tersebut"
            content = chat.str_message \
                if len(chat.str_message) < 1000 else f"{chat.str_message[:1000]}....(Sampai akhir)"
            reply = f"Konten : \n{content} \n\nAnalisa : \n{fact}"

        # log users
        user = BotUser(
            user_id=chat.sender_id,
            name=chat.sender
        )
        self.log_users(user)

        return reply

    @staticmethod
    def magic_keyword(keyword: str):
        magic_reply: str
        logging.info("Magic keyword : %s", keyword)
        if keyword == "!crawl":
            try:
                logging.info("Start adhoc crawling..")
                crawler.start()
                magic_reply = "Successfully crawled 10 articles"
                logging.debug("Clearing lru_cache..")
                get_cosine_sim.clear_cache()
            except Exception as ex:
                magic_reply = "Error crawling message"
        else:
            magic_reply = "No such magic!"
        return magic_reply

    @staticmethod
    def log_users(user: BotUser):
        """ method for storing users to db"""
        from datetime import datetime
        chat_timestamp = datetime.now().strftime("%Y-%m-%d %H:%M:%S")
        log: list = [user.user_id, user.name, chat_timestamp]
        try:
            logging.info("Storing user %s", log)
            conn = TabayyunDB(Conf.BOT_DB)
            conn.connection.execute('INSERT INTO bot_user VALUES (?,?,?)', log)
        except Exception as e:
            logging.error("Error when inserting bot user : %s", e)



import telegram
from flask import Flask, request

from src.config.env import EnvConfig as Config
from src.view.telegram_bot import handler, start_command, help_command, contact_command

BOT = telegram.Bot(token=Config.BOT_TOKEN)
CERT = 'cert.pem'
CERT_KEY = 'key.pem'
CONTEXT = (CERT, CERT_KEY)
APP = Flask(__name__)


@APP.route('/tabayyun-bot', methods=['POST'])
def respond():
    # retrieve the message in JSON and then transform it to Telegram object
    update = telegram.Update.de_json(request.get_json(force=True), BOT)

    if update.message.text == "/start":
        start_command(update)
    elif update.message.text == "/help":
        help_command(update)
    elif update.message.text == "/contact":
        contact_command(update)
    else:
        handler(update)

    return 'ok'


@APP.route('/set_webhook', methods=['GET', 'POST'])
def set_webhook():
    s = BOT.setWebhook(Config.WEBHOOK_URL, certificate=open(CERT, 'rb'))
    if s:
        return f"Webhook setup OK : {Config.WEBHOOK_URL}"
    else:
        return f"Webhook setup failed : {Config.WEBHOOK_URL}"


@APP.route('/')
def index():
    return '<h1>Tabayyun Bot webhook is running</h1>'


def start():
    APP.run(threaded=True, port=Config.WEBHOOK_PORT, host=Config.WEBHOOK_HOST, ssl_context=CONTEXT)


if __name__ == '__main__':
    start()



import logging

import pandas as pd
from sqlalchemy import create_engine, Column, String, Text, Integer
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.pool import SingletonThreadPool

# Global Variables
BASE = declarative_base()

logging.basicConfig(level=logging.INFO, format='%(asctime)s - %(levelname)s - %(message)s')


class MafindoNews(BASE):
    __tablename__ = "mafindo_tbl"
    id = Column(Integer, primary_key=True)
    authors = Column(Integer)
    status = Column(Integer)
    classification = Column(String(100))
    title = Column(Text)
    content = Column(Text)
    fact = Column(Text)
    reference_link = Column(Text)
    source_issue = Column(Text)
    source_link = Column(Text)
    picture1 = Column(String(500))
    picture2 = Column(String(100))
    tanggal = Column(String(12))
    tags = Column(String(500))
    conclusion = Column(Text)


class BotUser(BASE):
    __tablename__ = "bot_user"
    user_id = Column(String, primary_key=True)
    name = Column(String, primary_key=True)
    chat_timestamp = Column(String, primary_key=True)


class TabayyunDB:
    # http://docs.sqlalchemy.org/en/latest/core/engines.html

    def __init__(self, sqlite_db):
        logging.debug("Set SQLite as the DB engine")
        self.db_engine = create_engine(f'sqlite:///{sqlite_db}', poolclass=SingletonThreadPool)
        self.connection = self.db_engine.connect()
        self.apply_schema()

    def apply_schema(self):
        logging.debug("Applying schemas..")
        BASE.metadata.create_all(self.db_engine)

    def execute_query(self, query=''):
        if query == '':
            return
        with self.connection as conn:
            try:
                conn.execute(query)
            except Exception as e:
                logging.error(e)

    def get_data(self, table='', query=''):
        query = query if query != '' else "SELECT * FROM '{}';".format(table)
        logging.debug(query)
        with self.connection as conn:
            try:
                result_df = pd.read_sql(query, conn)
            except Exception as e:
                logging.error(e)
        return result_df


if __name__ == "__main__":
    TabayyunDB('tabayyun.db').apply_schema()



""" Configuration """
import logging
import os


class EnvConfig:
    BOT_DB = os.environ.get('BOT_DATABASE')
    BOT_TOKEN = os.environ.get('BOT_TOKEN')
    WEBHOOK_HOST = os.environ.get('WEBHOOK_HOST')
    WEBHOOK_URL = os.environ.get('WEBHOOK_URL', '0.0.0.0')
    WEBHOOK_PORT = os.environ.get('WEBHOOK_PORT', 8443)
    MAFINDO_API_KEY = os.environ.get('MAFINDO_API_KEY')
    LOG_LEVEL = os.environ.get('BOT_LOG_LEVEL', logging.INFO)
    LOG_FORMAT = '%(asctime)s - %(levelname)s - %(message)s'



import logging
import os
from sqlite3 import IntegrityError

import pandas as pd
import requests
from sqlalchemy.exc import IntegrityError

from src.config.env import EnvConfig as Conf
from src.dataset.database import TabayyunDB

logging.basicConfig(
    format=Conf.LOG_FORMAT, level=Conf.LOG_LEVEL
)
result = []


def get_mafindo_data(api_key: str, limit: int) -> list:
    logging.info("Getting data from MAFINDO API..")
    mafindo_data = []
    url = 'https://yudistira.turnbackhoax.id/api/antihoax'
    data = {'key': api_key, 'limit': limit}
    try:
        req = requests.post(url, data=data)
        for response in req.json():
            mafindo_data.append(
                [response['id'],
                 response['authors'],
                 response['status'],
                 response['classification'],
                 response['title'],
                 response['content'],
                 response['fact'],
                 response['references'],
                 response['source_issue'],
                 response['source_link'],
                 response['picture1'],
                 response['picture2'],
                 response['tanggal'],
                 response['tags'],
                 response['conclusion']
                 ])
    except requests.RequestException as exp:
        logging.error(exp)
    return mafindo_data


def start(limit: int = 10):
    api_key = os.environ['MAFINDO_API_KEY']
    mafindo_data = get_mafindo_data(api_key, limit)

    engine = TabayyunDB(sqlite_db=Conf.BOT_DB).db_engine
    mafindo_df = pd.DataFrame(mafindo_data,
                              columns=['id', 'authors', 'status', 'classification', 'title', 'content', 'fact',
                                       'reference_link', 'source_issue', 'source_link', 'picture1', 'picture2',
                                       'tanggal',
                                       'tags', 'conclusion'])

    for i, _ in enumerate(mafindo_df.iterrows()):
        try:
            mafindo_df.iloc[[i]].to_sql('mafindo_tbl', con=engine, if_exists='append', index=False, chunksize=100)
        except IntegrityError:
            logging.warning("%s already exist, skip the row!", mafindo_df.iloc[i]['id'])
            continue


if __name__ == '__main__':
    start("tabayyun.db")



import logging
from functools import lru_cache
from warnings import simplefilter

from sklearn.feature_extraction.text import CountVectorizer
from sklearn.metrics.pairwise import cosine_similarity

from src.config.env import EnvConfig as Config
from src.dataset.database import TabayyunDB

logging.basicConfig(format=Config.LOG_FORMAT, level=Config.LOG_LEVEL)
# ignore all future warnings
simplefilter(action='ignore', category=FutureWarning)


def get_dataset():
    conn = TabayyunDB(Config.BOT_DB)
    df = conn.get_data(table="mafindo_tbl")
    return df


@lru_cache()
def get_vectors(text1, text2):
    vectorizer = CountVectorizer(text1, text2)
    vectorizer.fit([text1, text2])
    return vectorizer.transform([text1, text2]).toarray()


@lru_cache()
def get_cosine_sim(text: str):
    ds = get_dataset()
    fact = "ARTIKEL TIDAK DITEMUKAN"
    link = []
    for _, row in ds.iterrows():
        vectors_content = get_vectors(text.lower(), row['content'].lower())
        if len(text.split()) > 2:
            similarity_content = cosine_similarity(vectors_content)
            vectors_tittle = get_vectors(text.lower(), row['title'].lower())
            similarity_tittle = cosine_similarity(vectors_tittle)
            if similarity_tittle[0][1] > 0.70 or similarity_content[0][1] > 0.70:
                return {"fact": row['fact'], "classification": row['classification'],
                        "conclusion": row['conclusion'], "references": [row['reference_link']]}

        if text.lower() in row['content'].lower():
            fact = "Ditemukan beberapa artikel terkait topik tersebut" \
                if len(set(link)) > 1 else "Ditemukan artikel terkait topik tersebut"
            link.append(f"{row['title']} - {row['reference_link']}")

    logging.info("get_cosine_sim.cache_info : %s", get_cosine_sim.cache_info())
    return {"fact": fact, "references": link}


if __name__ == '__main__':
    # Test sample message
    msg = """[SALAH] Racikan Air Kelapa Muda, Jeruk Nipis, Garam, dan Madu dapat Membunuh Virus Covid-19"""
    get_cosine_sim(msg)
